package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.utility.EconomyHelper;
import org.bukkit.Material;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.PluginCommand;
import org.bukkit.entity.Player;

import static net.knarcraft.bookswithoutborders.BooksWithoutBorders.sendErrorMessage;
import static net.knarcraft.bookswithoutborders.BooksWithoutBordersSettings.getCommandColor;
import static net.knarcraft.bookswithoutborders.BooksWithoutBordersSettings.getSuccessColor;

/**
 * Command executor for the books without borders (bwb) command
 */
public class CommandBooksWithoutBorders implements CommandExecutor {

    private final BooksWithoutBorders booksWithoutBorders = BooksWithoutBorders.getInstance();

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        sender.sendMessage(getCommandColor() + "[] denote optional parameters");
        sender.sendMessage(getCommandColor() + "<> denote required parameters");
        sender.sendMessage(getCommandColor() + "In some cases, commands with required parameters can be called with no parameters");
        if (sender instanceof Player) {
            showPlayerCommands(sender);
        } else {
            showConsoleCommands(sender);
        }
        return true;
    }

    /**
     * Shows all commands available to the console
     *
     * @param sender <p>The console which sent the command</p>
     */
    private void showConsoleCommands(CommandSender sender) {
        sender.sendMessage(getCommandColor() + "Commands:");
        showCommandInfo("reload", sender);
        showCommandInfo("givePublicBook", sender);
        showCommandInfo("deletePublicBook", sender);
    }

    /**
     * Shows all commands available to the sending player
     *
     * @param sender <p>The player which sent the command</p>
     */
    private void showPlayerCommands(CommandSender sender) {
        //Lists all commands
        Material bookPriceType = BooksWithoutBorders.getBookPriceType();
        double bookPriceQuantity = BooksWithoutBorders.getBookPriceQuantity();
        if (booksWithoutBorders.booksHavePrice()) {
            if (bookPriceType != Material.AIR) {
                sendErrorMessage(sender, "[" + (int) bookPriceQuantity + " " + bookPriceType.toString() +
                        "(s) are required to create a book]");
            } else {
                sendErrorMessage(sender, "[" + EconomyHelper.getEconomy().format(bookPriceQuantity) +
                        " is required to create a book]");
            }
        }
        sender.sendMessage(getCommandColor() + "Commands:");

        showCommandInfo("loadBook", sender);
        showCommandInfo("loadPublicBook", sender);
        showCommandInfo("saveBook", sender);
        showCommandInfo("savePublicBook", sender);
        showCommandInfo("giveBook", sender);
        showCommandInfo("givePublicBook", sender);
        showCommandInfo("deleteBook", sender);
        showCommandInfo("deletePublicBook", sender);
        showCommandInfo("unsignBook", sender);
        showCommandInfo("copyBook", sender);
        showCommandInfo("encryptBook", sender);
        showCommandInfo("groupEncryptBook", sender);
        showCommandInfo("decryptBook", sender);
        showCommandInfo("setTitle", sender);
        showCommandInfo("setAuthor", sender);
        showCommandInfo("setLore", sender);
        showCommandInfo("setBookPrice", sender);
        showCommandInfo("reload", sender);
    }

    /**
     * Shows information about the given command
     *
     * @param command <p>The command to get information about</p>
     * @param sender  <p>The sender asking to see command info</p>
     */
    private void showCommandInfo(String command, CommandSender sender) {
        PluginCommand pluginCommand = BooksWithoutBorders.getInstance().getCommand(command);
        if (pluginCommand != null) {
            String permission = pluginCommand.getPermission();
            if (permission == null || sender.hasPermission(permission)) {
                sender.sendMessage("\n" + getCommandColor() +
                        pluginCommand.getUsage().replace("<command>", pluginCommand.getName()) + ": " +
                        getSuccessColor() + pluginCommand.getDescription());
            }
        }
    }

}
