package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import net.knarcraft.bookswithoutborders.utility.EconomyHelper;
import net.knarcraft.bookswithoutborders.utility.InputCleaningHelper;
import net.knarcraft.bookswithoutborders.utility.InventoryHelper;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.BookMeta;

import java.util.Objects;

/**
 * Command executor for the copy command
 */
public class CommandCopy implements CommandExecutor {

    private final BooksWithoutBorders booksWithoutBorders = BooksWithoutBorders.getInstance();

    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        if (!(sender instanceof Player player)) {
            BooksWithoutBorders.sendErrorMessage(sender, "This command can only be used by a player!");
            return false;
        }

        if (InventoryHelper.notHoldingOneWrittenBookCheck(player, "You must be holding a written book to copy it!",
                "You cannot copy two books at once!")) {
            return false;
        }

        if (args.length < 1) {
            BooksWithoutBorders.sendErrorMessage(player, "You must specify the number of copies to be made!");
            return false;
        }

        ItemStack heldBook = InventoryHelper.getHeldBook(player, true);

        try {
            int copies = Integer.parseInt(args[0]);
            if (copies > 0) {
                if (BooksWithoutBorders.getAuthorOnlyCopy() && !player.hasPermission("bookswithoutborders.bypassAuthorOnlyCopy")) {
                    if (!isAuthor(player, (BookMeta) Objects.requireNonNull(heldBook.getItemMeta())))
                        return false;
                }

                if (booksWithoutBorders.booksHavePrice() &&
                        !player.hasPermission("bookswithoutborders.bypassBookPrice") &&
                        EconomyHelper.cannotPayForBookPrinting(player, copies)) {
                    return false;
                }

                heldBook.setAmount(heldBook.getAmount() + copies);
                BooksWithoutBorders.sendSuccessMessage(player, "Book copied!");
                return true;
            }
        } catch (NumberFormatException ignored) {
        }
        BooksWithoutBorders.sendErrorMessage(player, "Book not copied!");
        BooksWithoutBorders.sendErrorMessage(player, "Number specified was invalid!");
        return false;
    }

    /**
     * Checks whether the given player is the author of a given book
     *
     * @param player <p>The player to check</p>
     * @param book   <p>The book to check</p>
     * @return <p>True if the player is the book's author</p>
     */
    private boolean isAuthor(Player player, BookMeta book) {
        String author = book.getAuthor();
        String playerName = InputCleaningHelper.cleanString(player.getName());
        if (author != null && playerName.equalsIgnoreCase(InputCleaningHelper.cleanString(author))) {
            return true;
        }

        BooksWithoutBorders.sendErrorMessage(player, "You must be the author of this book to use this command!");
        return false;
    }

}
