package net.knarcraft.bookswithoutborders.command;

import net.knarcraft.bookswithoutborders.BooksWithoutBorders;
import org.bukkit.Server;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.entity.Player;

import java.util.ArrayList;
import java.util.List;

/**
 * Tab completer for the save command
 */
public class GiveTabCompleter implements TabCompleter {

    private final BooksWithoutBorders booksWithoutBorders = BooksWithoutBorders.getInstance();

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String alias, String[] args) {
        Server server = booksWithoutBorders.getServer();
        List<String> playerNames = new ArrayList<>();
        for (Player player : server.getOnlinePlayers()) {
            playerNames.add(player.getName());
        }
        List<String> booleans = new ArrayList<>();
        booleans.add("true");
        booleans.add("false");
        List<String> numbers = new ArrayList<>();
        numbers.add("1");
        numbers.add("2");
        numbers.add("3");
        numbers.add("4");
        numbers.add("5");
        List<String> boolAndNumbers = new ArrayList<>();
        boolAndNumbers.addAll(booleans);
        boolAndNumbers.addAll(numbers);

        int argumentCount = args.length;

        if (argumentCount > 2) {
            //Don't continue with autocompletion if the recipient is invalid
            if (server.getPlayer(args[1]) == null) {
                return null;
            }
        }

        if (argumentCount == 1) {
            //Return list of books
            return BooksWithoutBorders.getAvailableBooks(sender, false);
        } else if (argumentCount == 2) {
            //Return online players
            return playerNames;
        } else if (argumentCount == 3) {
            //Number of copies
            return boolAndNumbers;
        } else if (argumentCount == 4) {
            //Signed
            try {
                Integer.parseInt(args[2]);
                return booleans;
            } catch (NumberFormatException e) {
                return null;
            }
        }
        return null;
    }

}
